@@ -21,7 +21,7 @@ use goblin::{
     elf::{header::*, reloc::*, section_header::*, Elf},
     error::Error as GoblinError,
 };
-use std::{collections::BTreeMap, fmt::Debug, mem, ops::Range, str};
+use std::{collections::BTreeMap, fmt::Debug, mem, ops::Range, pin::Pin, str};
 
 /// Error definitions
 #[derive(Debug, thiserror::Error, PartialEq, Eq)]
@@ -68,6 +68,9 @@ pub enum ElfError {
     /// Read-write data not supported
     #[error("Found .bss section in ELF, read-write data not supported")]
     BssNotSupported,
+    /// Read-write data not supported
+    #[error("Found writable section ({0}) in ELF, read-write data not supported")]
+    WritableSectionNotSupported(String),
     /// Relocation failed, no loadable section contains virtual address
     #[error("Relocation failed, no loadable section contains virtual address {0:#x}")]
     AddressOutsideLoadableSection(u64),
@@ -282,8 +285,9 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
     }
 
     /// JIT compile the executable
-    pub fn jit_compile(&mut self) -> Result<(), EbpfError<E>> {
-        self.compiled_program = Some(JitProgram::<E, I>::new(self)?);
+    pub fn jit_compile(executable: &mut Pin<Box<Self>>) -> Result<(), EbpfError<E>> {
+        // TODO: Turn back to `executable: &mut self` once Self::report_unresolved_symbol() is gone
+        executable.compiled_program = Some(JitProgram::<E, I>::new(executable)?);
         Ok(())
     }
 
@@ -378,7 +382,7 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
         let elf = Elf::parse(bytes)?;
         let mut elf_bytes = AlignedMemory::new_with_data(bytes, ebpf::HOST_ALIGN);
 
-        Self::validate(&elf, elf_bytes.as_slice())?;
+        Self::validate(&config, &elf, elf_bytes.as_slice())?;
 
         // calculate the text section info
         let text_section = Self::get_section(&elf, ".text")?;
@@ -536,7 +540,7 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
     }
 
     /// Validates the ELF
-    pub fn validate(elf: &Elf, elf_bytes: &[u8]) -> Result<(), ElfError> {
+    pub fn validate(config: &Config, elf: &Elf, elf_bytes: &[u8]) -> Result<(), ElfError> {
         if elf.header.e_ident[EI_CLASS] != ELFCLASS64 {
             return Err(ElfError::WrongClass);
         }
@@ -569,8 +573,14 @@ impl<E: UserDefinedError, I: InstructionMeter> Executable<E, I> {
         }
 
         for section_header in elf.section_headers.iter() {
-            if let Some(this_name) = elf.shdr_strtab.get_at(section_header.sh_name) {
-                if this_name.starts_with(".bss") {
+            if let Some(name) = elf.shdr_strtab.get_at(section_header.sh_name) {
+                if config.reject_all_writable_sections
+                    && (name.starts_with(".bss")
+                        || (section_header.is_writable()
+                            && (name.starts_with(".data") && !name.starts_with(".data.rel"))))
+                {
+                    return Err(ElfError::WritableSectionNotSupported(name.to_owned()));
+                } else if name == ".bss" {
                     return Err(ElfError::BssNotSupported);
                 }
             }
@@ -862,28 +872,30 @@ mod test {
             .expect("failed to read elf file");
         let mut parsed_elf = Elf::parse(&bytes).unwrap();
         let elf_bytes = bytes.to_vec();
+        let config = Config::default();
 
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect("validation failed");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect("validation failed");
         parsed_elf.header.e_ident[EI_CLASS] = ELFCLASS32;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect_err("allowed bad class");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect_err("allowed bad class");
         parsed_elf.header.e_ident[EI_CLASS] = ELFCLASS64;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect("validation failed");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect("validation failed");
         parsed_elf.header.e_ident[EI_DATA] = ELFDATA2MSB;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect_err("allowed big endian");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect_err("allowed big endian");
         parsed_elf.header.e_ident[EI_DATA] = ELFDATA2LSB;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect("validation failed");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect("validation failed");
         parsed_elf.header.e_ident[EI_OSABI] = 1;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect_err("allowed wrong abi");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect_err("allowed wrong abi");
         parsed_elf.header.e_ident[EI_OSABI] = ELFOSABI_NONE;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect("validation failed");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect("validation failed");
         parsed_elf.header.e_machine = EM_QDSP6;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect_err("allowed wrong machine");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes)
+            .expect_err("allowed wrong machine");
         parsed_elf.header.e_machine = EM_BPF;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect("validation failed");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect("validation failed");
         parsed_elf.header.e_type = ET_REL;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect_err("allowed wrong type");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect_err("allowed wrong type");
         parsed_elf.header.e_type = ET_DYN;
-        ElfExecutable::validate(&parsed_elf, &elf_bytes).expect("validation failed");
+        ElfExecutable::validate(&config, &parsed_elf, &elf_bytes).expect("validation failed");
     }
 
     #[test]
@@ -1180,4 +1192,39 @@ mod test {
         ElfExecutable::load(Config::default(), &elf_bytes, syscall_registry())
             .expect("validation failed");
     }
+
+    #[test]
+    #[should_panic(expected = r#"validation failed: WritableSectionNotSupported(".data")"#)]
+    fn test_writable_data_section() {
+        let elf_bytes =
+            std::fs::read("tests/elfs/writable_data_section.so").expect("failed to read elf file");
+
+        assert!(ElfExecutable::load(Config::default(), &elf_bytes, syscall_registry()).is_ok());
+
+        ElfExecutable::load(
+            Config {
+                reject_all_writable_sections: true,
+                ..Config::default()
+            },
+            &elf_bytes,
+            syscall_registry(),
+        )
+        .expect("validation failed");
+    }
+
+    #[test]
+    #[should_panic(expected = r#"validation failed: WritableSectionNotSupported(".bss")"#)]
+    fn test_bss_section() {
+        let elf_bytes =
+            std::fs::read("tests/elfs/bss_section.so").expect("failed to read elf file");
+        ElfExecutable::load(
+            Config {
+                reject_all_writable_sections: true,
+                ..Config::default()
+            },
+            &elf_bytes,
+            syscall_registry(),
+        )
+        .expect("validation failed");
+    }
 }
